# coding: utf-8

import scipy, numpy, threading
from scipy import weave
import Tkinter
import ttk #@UnresolvedImport
import pylaf
from pylaf import Component, Port, Panel, Entry, TableBuilder
from pylaf.vtkext import numpy_to_vtk, UIntScalarStructuredPoints, StructuredPoints

def calchx(yee,dt):
    code =\
    """
    #define mu 1.256637061
    #define Vhx(i,j,k) (hx+(i)*Nhx[1]*Nhx[2]+(j)*Nhx[2]+k)
    #define Vey(i,j,k) (ey+(i)*Ney[1]*Ney[2]+(j)*Ney[2]+k)
    #define Vez(i,j,k) (ez+(i)*Nez[1]*Nez[2]+(j)*Nez[2]+k)
    for (int k = 0; k < Nhx[2]; k++)
        for (int j = 0; j < Nhx[1]; j++)
            for (int i = 0; i < Nhx[0]; i++) {
                *Vhx(i,j,k) +=
                    - dt/mu/dy*(*Vez(i,j+1,k  )-*Vez(i,j,k))
                    + dt/mu/dz*(*Vey(i,j  ,k+1)-*Vey(i,j,k));
            }
    """
    ex, ey, ez = yee.efield
    hx, hy, hz = yee.hfield
    dx, dy, dz = yee.delta
    weave.inline(code,['hx','ey','ez','dy','dz','dt'])

def calchy(yee,dt):
    code =\
    """
    #define mu 1.256637061
    #define Vex(i,j,k) (ex+(i)*Nex[1]*Nex[2]+(j)*Nex[2]+k)
    #define Vhy(i,j,k) (hy+(i)*Nhy[1]*Nhy[2]+(j)*Nhy[2]+k)
    #define Vez(i,j,k) (ez+(i)*Nez[1]*Nez[2]+(j)*Nez[2]+k)
    for (int k = 0; k < Nhy[2]; k++)
        for (int j = 0; j < Nhy[1]; j++)
            for (int i = 0; i < Nhy[0]; i++) {
                *Vhy(i,j,k) +=
                    - dt/mu/dz*(*Vex(i  ,j,k+1)-*Vex(i,j,k))
                    + dt/mu/dx*(*Vez(i+1,j,k  )-*Vez(i,j,k));
            }
    """
    ex, ey, ez = yee.efield
    hx, hy, hz = yee.hfield
    dx, dy, dz = yee.delta
    weave.inline(code,['ex','hy','ez','dz','dx','dt'])

def calchz(yee,dt):
    code =\
    """
    #define mu 1.256637061
    #define Vex(i,j,k) (ex+(i)*Nex[1]*Nex[2]+(j)*Nex[2]+k)
    #define Vey(i,j,k) (ey+(i)*Ney[1]*Ney[2]+(j)*Ney[2]+k)
    #define Vhz(i,j,k) (hz+(i)*Nhz[1]*Nhz[2]+(j)*Nhz[2]+k)
    for (int k = 0; k < Nhz[2]; k++)
        for (int j = 0; j < Nhz[1]; j++)
            for (int i = 0; i < Nhz[0]; i++) {
                *Vhz(i,j,k) +=
                    - dt/mu/dx*(*Vey(i+1,j  ,k)-*Vey(i,j,k))
                    + dt/mu/dy*(*Vex(i  ,j+1,k)-*Vex(i,j,k));
            }
    """
    ex, ey, ez = yee.efield
    hx, hy, hz = yee.hfield
    dx, dy, dz = yee.delta
    weave.inline(code,['ex','ey','hz','dx','dy','dt'])

def calcex(yee,dt):
    code =\
    """
    #define ep 0.000008854187817
    #define Vex(i,j,k) (ex+(i)*Nex[1]*Nex[2]+(j)*Nex[2]+k)
    #define Vhy(i,j,k) (hy+(i)*Nhy[1]*Nhy[2]+(j)*Nhy[2]+k)
    #define Vhz(i,j,k) (hz+(i)*Nhz[1]*Nhz[2]+(j)*Nhz[2]+k)
    #define Vepr(i,j,k) (epr+(i)*Nepr[1]*Nepr[2]+(j)*Nepr[2]+k)
    for (int k = 1; k < Nex[2]-1; k++)
        for (int j = 1; j < Nex[1]-1; j++)
            for (int i = 0; i < Nex[0]; i++) {
                double er = (*Vepr(i,j,k)+*Vepr(i,j,k-1)+*Vepr(i,j-1,k-1)+*Vepr(i,j-1,k))/4;
                *Vex(i,j,k) +=
                    + dt/ep/er/dy*(*Vhz(i,j,k)-*Vhz(i,j-1,k  ))
                    - dt/ep/er/dz*(*Vhy(i,j,k)-*Vhy(i,j  ,k-1));
            }
    """
    ex, ey, ez = yee.efield
    hx, hy, hz = yee.hfield
    dx, dy, dz = yee.delta
    epr        = yee.epr
    weave.inline(code,['ex','hy','hz','dy','dz','dt','epr'])
    
def pcalcex(yee,dt):
    ep = 0.000008854187817
    ex, ey, ez = yee.efield
    hx, hy, hz = yee.hfield
    dx, dy, dz = yee.delta
    epr        = yee.epr
    er = (epr[:,1:,1:]+epr[:,1:,:-1]+epr[:,:-1,:-1]+epr[:,:-1,1:]) / 4.
    ex[:,1:-1,1:-1] += dt / ep / er / dy * (hz[:,1:,1:-1] - hz[:,:-1,1:-1])\
                     - dt / ep / er / dz * (hy[:,1:-1,1:] - hy[:,1:-1,:-1])
                     
def calcey(yee,dt):
    code =\
    """
    #define ep 0.000008854187817
    #define Vhx(i,j,k) (hx+(i)*Nhx[1]*Nhx[2]+(j)*Nhx[2]+k)
    #define Vey(i,j,k) (ey+(i)*Ney[1]*Ney[2]+(j)*Ney[2]+k)
    #define Vhz(i,j,k) (hz+(i)*Nhz[1]*Nhz[2]+(j)*Nhz[2]+k)
    #define Vepr(i,j,k) (epr+(i)*Nepr[1]*Nepr[2]+(j)*Nepr[2]+k)
    for (int k = 1; k < Ney[2]-1; k++)
        for (int j = 0; j < Ney[1]; j++)
            for (int i = 1; i < Ney[0]-1; i++) {
                double er = (*Vepr(i,j,k)+*Vepr(i,j,k-1)+*Vepr(i-1,j,k-1)+*Vepr(i-1,j,k))/4;
                *Vey(i,j,k) +=
                    + dt/ep/er/dz*(*Vhx(i,j,k)-*Vhx(i  ,j,k-1))
                    - dt/ep/er/dx*(*Vhz(i,j,k)-*Vhz(i-1,j,k  ));
            }
    """
    ex, ey, ez = yee.efield
    hx, hy, hz = yee.hfield
    dx, dy, dz = yee.delta
    epr        = yee.epr
    weave.inline(code,['hx','ey','hz','dz','dx','dt','epr'])

def calcez(yee,dt):
    code =\
    """
    #define ep 0.000008854187817
    #define Vhx(i,j,k) (hx+(i)*Nhx[1]*Nhx[2]+(j)*Nhx[2]+k)
    #define Vhy(i,j,k) (hy+(i)*Nhy[1]*Nhy[2]+(j)*Nhy[2]+k)
    #define Vez(i,j,k) (ez+(i)*Nez[1]*Nez[2]+(j)*Nez[2]+k)
    #define Vepr(i,j,k) (epr+(i)*Nepr[1]*Nepr[2]+(j)*Nepr[2]+k)
    for (int k = 0; k < Nez[2]; k++)
        for (int j = 1; j < Nez[1]-1; j++)
            for (int i = 1; i < Nez[0]-1; i++) {
                double er = (*Vepr(i,j,k)+*Vepr(i,j-1,k)+*Vepr(i-1,j-1,k)+*Vepr(i-1,j,k))/4;
                *Vez(i,j,k) +=
                    + dt/ep/er/dx*(*Vhy(i,j,k)-*Vhy(i-1,j  ,k))
                    - dt/ep/er/dy*(*Vhx(i,j,k)-*Vhx(i  ,j-1,k));
            }
    """
    ex, ey, ez = yee.efield
    hx, hy, hz = yee.hfield
    dx, dy, dz = yee.delta
    epr        = yee.epr
    weave.inline(code,['hx','hy','ez','dx','dy','dt','epr'])
    
class StructuredYee:
    def __init__(self,size=(30,30,30),delta=(10.,10.,10.),dt=1.):
        self.size         = size
        self.shape        = size
        self.delta        = delta
        self.dt           = dt
        self.current_time = 0.
        self.clear_fields()
        self.epr    = scipy.ones(size,dtype=scipy.double)
    def clear_fields(self):
        size, opts = self.size, {'dtype':scipy.double}
        hx    = scipy.zeros([size[0]+1,size[1]  ,size[2]  ],**opts)
        hy    = scipy.zeros([size[0]  ,size[1]+1,size[2]  ],**opts)
        hz    = scipy.zeros([size[0]  ,size[1]  ,size[2]+1],**opts)
        ex    = scipy.zeros([size[0]  ,size[1]+1,size[2]+1],**opts)
        ey    = scipy.zeros([size[0]+1,size[1]  ,size[2]+1],**opts)
        ez    = scipy.zeros([size[0]+1,size[1]+1,size[2]  ],**opts)
        self.hfield = [hx,hy,hz]
        self.efield = [ex,ey,ez]
    def courant(self):
        max_epr = self.epr.max()
        vc = 299.792458 / scipy.sqrt(max_epr)
        dx, dy, dz = self.delta
        tc = 1./(vc * scipy.sqrt(1./dx**2 + 1./dy**2 + 1./dz**2))
        return tc
        
class YeeUIntScalarStructuredPoints(UIntScalarStructuredPoints):
    def __init__(self,**kw):
        UIntScalarStructuredPoints.__init__(self,**kw)
    def reset(self):
        UIntScalarStructuredPoints.reset(self)
        yee = self.sigin.get()
        try: yee.delta
        except AttributeError: return
        dz, dy, dx = yee.delta
        if not self.GetSpacing() == (dx, dy, dz):
            self.config(Spacing=(dx, dy, dz))
    def update(self,yee):
        ex, ey, ez = yee.efield
        ex = (ex[:,:-1,:-1]+ex[:,:-1,1:]+ex[:,1:,1:]+ex[:,1:,:-1]) / 4.
        ey = (ey[:-1,:,:-1]+ey[:-1,:,1:]+ey[1:,:,1:]+ey[1:,:,:-1]) / 4.
        ez = (ez[:-1,:-1,:]+ez[:-1,1:,:]+ez[1:,1:,:]+ez[1:,:-1,:]) / 4.
        return scipy.sqrt(ex*ex+ey*ey+ez*ez)
        
class YeeStructuredPoints(StructuredPoints):
    def __init__(self,**kw):
        StructuredPoints.__init__(self,**kw)
    def reset(self):
        StructuredPoints.reset(self)
        yee = self.sigin.get()
        try: yee.delta
        except AttributeError: pass
        else:
            dz, dy, dx = yee.delta
            self.config(Spacing=(dx, dy, dz))
    def shape(self,yee):
        if not len(yee.shape) == 3: return StructuredPoints.shape(self,yee)
        i, j, k = yee.shape
        return i, j, k, 3
    def update_scalars(self):
        k, j, i, c = self.vectorData.shape
        self.scalarData[:,:,:] = self.vectorData[:,:,:,0] ** 2
        for l in range(1,c):
            self.scalarData[:,:,:] += (self.vectorData[:,:,:,l] ** 2)
        self.scalarData[:,:,:] = numpy.sqrt(self.scalarData)
        
class YeeEStructuredPoints(YeeStructuredPoints):
    def __init__(self,**kw):
        YeeStructuredPoints.__init__(self,**kw)
    def update_vectors(self,vector,yee):
        ex, ey, ez = yee.efield
        vector[:,:,:,0] = (ex[:,:-1,:-1]+ex[:,:-1,1:]+ex[:,1:,1:]+ex[:,1:,:-1]) / 4.
        vector[:,:,:,1] = (ey[:-1,:,:-1]+ey[:-1,:,1:]+ey[1:,:,1:]+ey[1:,:,:-1]) / 4.
        vector[:,:,:,2] = (ez[:-1,:-1,:]+ez[:-1,1:,:]+ez[1:,1:,:]+ez[1:,:-1,:]) / 4.
        
class YeeHStructuredPoints(YeeStructuredPoints):
    def __init__(self,**kw):
        YeeStructuredPoints.__init__(self,**kw)
    def update_vectors(self,vector,yee):
        hx, hy, hz = yee.hfield
        vector[:,:,:,0] = (hx[:-1,:,:]+hx[1:,:,:]) / 2.
        vector[:,:,:,1] = (hy[:,:-1,:]+hy[:,1:,:]) / 2.
        vector[:,:,:,2] = (hz[:,:,:-1]+hz[:,:,1:]) / 2.
        z, y, x, c = vector.shape
        self.GetPointData().SetVectors(numpy_to_vtk(vector.reshape(z * y * x, c)))
        
class Plugin:
    def __init__(self,**kw):
        self.dataset = Port(None)
        self.config(**kw)
    def config(self,**kw):
        if kw == None: return
        if kw.has_key('dataset'): kw['dataset'].link(self.dataset)
    def __call__(self): pass

class UpdatingTime(Plugin):
    def __init__(self,**kw):
        self.counter = Port(None)
        self.offset  = Port(0.)
        Plugin.__init__(self,**kw)
    def config(self,**kw):
        Plugin.config(self,**kw)
        if kw.has_key('counter'): kw['counter'].link(self.counter)
        if kw.has_key('offset') : self.offset.set(kw['offset'])
    def __call__(self):
        dataset = self.dataset.get()
        dt      = dataset.dt
        dataset.current_time = dt * (self.counter.get() + self.offset.get())
        
#import time

class UpdatingEField(Plugin):
    def __call__(self):
        dataset = self.dataset.get()
        dt      = dataset.dt
        #st = time.time()
        pcalcex(dataset,dt)
        #et = time.time()
        #print ('Running Time (pure) = %f [sec]' % (et-st))
        #st = time.time()
        calcey(dataset,dt)
        #et = time.time()
        #print ('Running Time (weave) = %f [sec]' % (et-st))
        calcez(dataset,dt)
        
class UpdatingHField(Plugin):
    def __call__(self):
        dataset = self.dataset.get()
        dt      = dataset.dt
        calchx(dataset,dt); calchy(dataset,dt); calchz(dataset,dt)

class AlgorithmView(ttk.Treeview):
    def __init__(self,master=None,name=None,**kw):
        ttk.Treeview.__init__(self,master,name=name)
        self.config(**kw)
        self.loop()
    def loop(self):
        self.populate()
        self.after(500,self.loop)
    def populate(self,parent=''):
        pass
        
class Fdtd(Component):
    def __init__(self,master=None,name=None,**kw):
        Component.__init__(self,master,name)
        self.yee        = self.sigout = Port(None)
        self.maxiter    = Port(1000)
        self.counter    = Port(0)
        self.finish     = Port(None)
        self.initialize = []
        self.sequence   = []
        self.config(**kw)
    def config(self,**kw):
        if kw == None: return
        if kw.has_key('dataset'): self.yee.set(kw['dataset'])
    def start(self):
        for o in self.initialize: o()
        thread = threading.Thread(target=self.run)
        thread.setDaemon(True)
        thread.start()
    def cancel(self):
        self.canceled = None
    def run(self):
        counter = self.counter.get()
        while counter < self.maxiter.get():
            try: self.canceled
            except AttributeError: pass
            else:
                del self.canceled
                break
            self.step()
            counter = self.counter.get()
        self.finish.set(None)
    def step(self):
        for o in self.sequence: o()
        self.yee.subject.notify(pylaf.EVENT_SET)
        self.counter.set(self.counter.get() + 1)
    class Control(Panel):
        def __init__(self,master=None,cnf={},**kw):
            Panel.__init__(self,master,cnf,**kw)
            #
            Tkinter.Label(self,text=u'カウンタ(最大反復数)').grid(row=0,column=0)
            f = Tkinter.Frame(self); f.grid(row=0,column=1)
            self.grid_rowconfigure(0,weight=0)
            pylaf.Label(f,name='counter',width=8).pack(side=Tkinter.LEFT,expand=True,fill=Tkinter.X)
            Tkinter.Label(f,text='(').pack(side=Tkinter.LEFT,expand=True,fill=Tkinter.X)
            Entry(f,name='maxiter',width=8).pack(side=Tkinter.LEFT,expand=True,fill=Tkinter.X)
            Tkinter.Label(f,text=')').pack(side=Tkinter.LEFT,expand=True,fill=Tkinter.X)
            #
            b, l, e = TableBuilder(self), Tkinter.Label, Entry
            b.add(Tkinter.Button,name='calculate',text='Calculate',command=self.calculate).grid(columnspan=2,sticky=Tkinter.W+Tkinter.E)
            b.add(Tkinter.Button,name='clear',text='Clear Fields',command=self._clear).grid(columnspan=2,sticky=Tkinter.W+Tkinter.E)
            b.add(AlgorithmView,name='sequence').grid(columnspan=2)
            #b.add(AlgorithmView,name='hstep').grid(columnspan=2)
            self.finish = Port(None).bind(self._finished)
            self.grid_rowconfigure(1,weight=0)
            self.grid_rowconfigure(2,weight=0)
        def assign(self,component):
            Panel.assign(self,component)
            
        def calculate(self):
            calculate = self.children['calculate']
            text = calculate.config('text')[-1]
            if   text == 'Calculate':
                self.component().start()
                calculate.config(text='Cancel')
            elif text == 'Cancel':
                self.component().cancel()
        def _finished(self):
            calculate = self.children['calculate']
            text = calculate.config('text')[-1]
            if text == 'Cancel':
                calculate.config(text='Calculate')
        def _clear(self):
            c = self.component()
            c.counter.set(0)
            c.yee.get().clear_fields()
            c.yee.subject.notify(pylaf.EVENT_SET)
            
dummy_yee = StructuredYee(size=(1,1,1))
calchx(dummy_yee,dummy_yee.dt)
calchy(dummy_yee,dummy_yee.dt)
calchz(dummy_yee,dummy_yee.dt)
calcex(dummy_yee,dummy_yee.dt)
calcey(dummy_yee,dummy_yee.dt)
calcez(dummy_yee,dummy_yee.dt)
dummy_yee = None
#



# coding: utf-8

import Tkinter, gc
import matplotlib
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
matplotlib.use('Agg',warn=False)
#matplotlib.rcParams['figure.facecolor'] = 'white'
#matplotlib.rcParams['figure.edgecolor'] = 'white'
#matplotlib.rcParams['font.family'] = 'sans-serif'
matplotlib.rcParams['font.size']   = 9.0

class Frame(Tkinter.Frame):
    '''
    軽量プロットウィジェット
    tkaggバックエンドでのメモリ解放に関する不具合を修正したプロッタ
    '''
    def __init__(self,master=None,figsize=(5,4),dpi=75,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.figure = figure = Figure(figsize=figsize,dpi=dpi)
        # === make canvas object ===
        old = list(master.winfo_toplevel()._tclCommands) # tclコマンドを控える
        self.canvas = canvas = FigureCanvasTkAgg(figure,master=self) # ここでscroll_event_windowsとfilter_destroyがrootに追加される
        self._root_tclCommands = [] # destroy時に追加分だけ破棄するためのidを保持する
        for com in master.winfo_toplevel()._tclCommands:
            if old.count(com): continue
            self._root_tclCommands.append(com)
        # ===
        canvas.get_tk_widget().pack()
    def destroy_figure_canvas_axes(self):
        # rootにバインドされているイベントを削除する
        root = self.canvas._tkcanvas.winfo_toplevel()
        for name in self._root_tclCommands:
            root.deletecommand(name)
        self._root_tclCommands = []
        # figureオブジェクトを削除する
        self.figure.clear() # 生成されているaxesオブジェクトをクリアする
        self.canvas.figure = None # FigureCanvasTkAggからの参照を削除する
        self.figure.figurePatch = None # Rectangleオブジェクトを削除して逆参照をクリアする
        self.figure.patch = None # Rectangleオブジェクトを削除して逆参照をクリアする
        self.figure = None # MPLFrameからの参照をクリアする
        # canvasオブジェクトを削除する
        self.canvas._tkcanvas.destroy(); self.canvas._tkcanvas = None # canvasからtkcanvasを破棄する
        for cid in range(self.canvas.callbacks._cid): self.canvas.mpl_disconnect(cid+1) # コールバックを削除する
        matplotlib.backend_bases.LocationEvent.lastevent = None # <- これはまだ必要かどうかわからない。クリックしたら解放されなくなるバグの修正
        self.canvas = None # MPLFrameからの参照をクリアする
        gc.collect()
    def destroy(self):
        self.destroy_figure_canvas_axes()
        Tkinter.Frame.destroy(self)

# class BasePlot(Frame):
# class Rectlinear(BasePlot): PROJECTION='rectlinear'
# class Polar(BasePlot): PROJECTION='polar'
class BasePlot(Frame):
    def __init__(self,master=None,figsize=(5,4),dpi=75,align=111,projection='rectilinear',cnf={},**kw):
        Frame.__init__(self,master,figsize,dpi,cnf,**kw)
        self.ax = self.figure.add_subplot(align,projection=projection) # １つの矩形プロットを追加する
    def resize(self,figsize=(5,4),dpi=75,align=111,projection='rectilinear'):
        self.destroy_figure_canvas_axes()
        self.figure = figure = Figure(figsize=figsize,dpi=dpi)
        # === make canvas object ===
        master = self.master
        old = list(master.winfo_toplevel()._tclCommands) # tclコマンドを控える
        self.canvas = canvas = FigureCanvasTkAgg(figure,master=self) # ここでscroll_event_windowsとfilter_destroyがrootに追加される
        self._root_tclCommands = [] # destroy時に追加分だけ破棄するためのidを保持する
        for com in master.winfo_toplevel()._tclCommands:
            if old.count(com): continue
            self._root_tclCommands.append(com)
        # ===
        canvas.get_tk_widget().pack()
        self.ax = self.figure.add_subplot(align,projection=projection)
    def _clear_lines(self): # axes.clear
        while len(self.ax.lines): # 残っているすべてのラインについて
            line = self.ax.lines[-1] # 最終ラインへの参照
            self.ax.lines.remove(line) # ラインの削除
            line._lineFunc = None # Line2Dの循環参照を解放する
            #line = weakref.ref(line); print line # オブジェクトが削除されたか確認するデバッグコード
            # === 削除しないで！ ===
            # ラインの解放にトラブルが発生したら以下のコードを試してみる
            # たとえば、スミスチャートの実装時などに問題がでるかも
            #for key,item in self.ax.xaxis.callbacks.callbacks['units'].items(): # コールバックを解除する
            #    if item == line.recache:
            #        del self.ax.xaxis.callbacks.callbacks['units'][key]
            #for key,item in self.ax.yaxis.callbacks.callbacks['units'].items(): # コールバックを解除する
            #    if item == line.recache:
            #        del self.ax.yaxis.callbacks.callbacks['units'][key]
            # ===================
        self.ax.ignore_existing_data_limits = True # dataLimをアップデートする
        self.ax._get_lines.set_color_cycle() # カラーサイクルを初期状態に戻す
    def _clear_patch(self):
        while len(self.ax.patches): # きちんとパッチが解放されることを確認した
            patch = self.ax.patches[-1]
            self.ax.patches.remove(patch)
    def clear(self):
        self._clear_lines()
        self._clear_patch()
#    def savefig(self):
#        fname = tkFileDialog.asksaveasfilename(title='Enter Filename')
#        self.ax.figure.savefig(fname)
#    def load(self):
#        from numpy import genfromtxt
#        fname = tkFileDialog.askopenfilename(title='Enter Filename')
#        data = genfromtxt(fname,delimiter=',').transpose()
#        self.sig_in.set(data)
    def destroy(self):
        self.ax = None
        Frame.destroy(self)
        
# ３次元についてはVTKを利用するかも
class ThreeD(Frame):
    def __init__(self,master=None,figsize=(5,4),dpi=75,align=111,cnf={},**kw):
        Frame.__init__(self,master,figsize,dpi,cnf,**kw)
        self.ax = self.figure.add_subplot(align,projection='rectilinear') # １つの矩形プロットを追加する
    def plot(self): pass
    def scatter(self): pass
    def wireframe(self): pass
    def surface(self): pass
    def destroy(self):
        self.ax = None
        Frame.destroy(self)



# coding: utf-8

import Tkinter
import ttk #@UnresolvedImport
import core, gui, mpl, uuid, inspect

PRIMARY_PORTS = {gui.Button:          'value',
                 gui.Entry:           'value',
                 gui.Label:           'value',
                 gui.EntryTable:      'value',
                 gui.LabelTable:      'value',
                 gui.MultipleEntries: 'value',
                 mpl.BasePlot:        'sigin',
                 mpl.Oscilloscope:    'sigin',
                 mpl.Gauss:           'sigin',
                 mpl.Spectrum:        'sigin'}

class FactoryPort:
    CONTEXT  = []
    SEQUENCE = None
    ITEMS    = None
    @classmethod
    def new_context(cls):
        cls.CONTEXT.insert(0,(cls.SEQUENCE,cls.ITEMS))
        cls.SEQUENCE = []
        cls.ITEMS    = {}
    @classmethod
    def resume(cls):
        context = cls.CONTEXT.pop(0)
        cls.SEQUENCE,cls.ITEMS = context
    @classmethod
    def reset(cls):
        cls.SEQUENCE = None
        cls.ITEMS    = None
    def __init__(self):
        self.reset()
    def gui(self,widget_klass,cnf={},*args,**kw):
        # 
        if FactoryPort.SEQUENCE == None:
            FactoryPort.SEQUENCE = []
            FactoryPort.ITEMS    = {}
        # オプションのマージ
        for key in cnf:
            kw[key] = cnf[key]
        #
        FactoryPort.SEQUENCE.append(self)
        FactoryPort.ITEMS[self] = {'widget_klass': widget_klass}
        #
        def option(master=None,**kw): return kw # masterは無効
        kw = option(**kw)
        if not kw:
            kw = {}
        FactoryPort.ITEMS[self]['widget_args']   = args
        FactoryPort.ITEMS[self]['widget_kwargs'] = kw
        return self
    def map(self,**kw):
        FactoryPort.ITEMS[self]['map'] = kw
        return self
    def grid(self,**kw):
        def option(row=0,column=0,rowspan=0,columnspan=0,**kw): return kw
        FactoryPort.ITEMS[self]['grid_option'] = option(**kw)
        return self
    def label(self,text='',label_klass=ttk.Label,**kw):
        kw['text'] = text
        FactoryPort.ITEMS[self]['label_option'] = kw
        FactoryPort.ITEMS[self]['label_klass']  = label_klass
        return self
    def label_grid(self,**kw):
        def option(row=0,column=0,rowspan=0,columnspan=0,**kw): return kw
        FactoryPort.ITEMS[self]['label_grid_option'] = option(**kw)
        return self

class Port(core.Port,FactoryPort):
    def __init__(self,*args,**kw):
        core.Port.__init__(self,*args,**kw)
        FactoryPort.__init__(self)

class Output(core.Output,FactoryPort):
    def __init__(self,*args,**kw):
        core.Output.__init__(self,*args,**kw)
        FactoryPort.__init__(self)

class Grid(gui.Grid):
    _ezcomponent = None
    _ezcounter   = 0
    @classmethod
    def make(cls,component):
        try: cls._ezlock
        except AttributeError: pass
        else:
            FactoryPort.reset()
            return
        def extract_name(port):
            members = inspect.getmembers(component)
            objs    = [obj for name,obj in members]
            if objs.count(port):
                index = objs.index(port)
                name  = members[index][0]
                return name
        def append(port):
            klass = FactoryPort.ITEMS[port]['widget_klass']
            name  = FactoryPort.ITEMS[port]['name']
            try:
                args = FactoryPort.ITEMS[port]['widget_args']
            except KeyError:
                args = ()
            try:
                kwargs = FactoryPort.ITEMS[port]['widget_kwargs']
            except KeyError:
                kwargs = {}
            option = (args,kwargs)
            #
            try:
                o = FactoryPort.ITEMS[port]['label_option']
            except KeyError:
                label = None
            else:
                try:
                    label = o['text']
                    del o['text']
                except KeyError:
                    label = None
            #
            try:
                map = FactoryPort.ITEMS[port]['map']
            except KeyError:
                map = {}
            if PRIMARY_PORTS.has_key(klass):
                if not map.has_key(PRIMARY_PORTS[klass]):
                    map[PRIMARY_PORTS[klass]] = name
            #
            try:
                geometry = FactoryPort.ITEMS[port]['grid_option']
            except KeyError:
                geometry = {}
            try:
                label_widget = FactoryPort.ITEMS[port]['label_option']
            except KeyError:
                label_widget = {}
            try:
                label_geometry = FactoryPort.ITEMS[port]['label_grid_option']
            except KeyError:
                label_geometry = {}
            cls._ezconfig.append((klass,
                                  name,
                                  option,
                                  label,
                                  map,
                                  geometry,
                                  label_widget,
                                  label_geometry))
        #
        cls._ezconfig = []
        # アトリビュート名とポートオブジェクトの対応マップを生成する
        for port in FactoryPort.SEQUENCE:
            name = extract_name(port)
            if name:
                FactoryPort.ITEMS[port]['name'] = name
        # アイテムを登録する
        for port in FactoryPort.SEQUENCE:
            append(port)
        FactoryPort.reset()
        # ファクトリの受付終了
        cls._ezlock    = True
        cls._ezcounter = 0
    @classmethod
    def _initialize(cls):
        # いちばん最初の呼び出しの際に生成ルールリストを用意する
        try: cls._ezconfig
        except AttributeError: cls._ezconfig = []
        try: cls._primary
        except AttributeError:
            cls._primary = {gui.Button:          'value',
                            gui.Entry:           'value',
                            gui.Label:           'value',
                            gui.EntryTable:      'value',
                            gui.LabelTable:      'value',
                            gui.MultipleEntries: 'value'}
    @classmethod
    def _append(cls,
                klass,
                name=None,
                label=None,
                map=None,
                geometry=None,
                label_widget=None,
                label_geometry=None,
                cnf=None,
                *args,
                **kwargs):
        # 呼び出しカウンタをインクリメントする
        cls._ezcounter += 1
        # ロックされていなければ生成ルールを追加する
        try: cls._ezlock
        except AttributeError:
            # デフォルトの辞書を生成する
            if map            == None: map            = {}
            if geometry       == None: geometry       = {}
            if label_widget   == None: label_widget   = {}
            if label_geometry == None: label_geometry = {}
            if cnf            == None: cnf            = {}
            # nameが未定義ならばuuidをnameとする
            if name == None: name = str(uuid.uuid4())
            # mapにデフォルトポートが含まれていなければ追加する
            if cls._primary.has_key(klass):
                if not map.has_key(cls._primary[klass]):
                    map[cls._primary[klass]] = name
            # ウィジェット生成オプションの処理
            for key in cnf: kwargs[key] = cnf[key] # kwargsにwidgetを上書き追加する
            for key in ('master'): # masterは無効にする
                if kwargs.has_key(key): del kwargs[key]
            option = (args,kwargs)
            # ウィジェット生成情報の登録
            cls._ezconfig.append((klass,name,option,label,map,geometry,label_widget,label_geometry)) # アイテムリストへ追加する
        # プライマリポートを返す
        klass = cls._ezconfig[cls._ezcounter - 1][0]
        map   = cls._ezconfig[cls._ezcounter - 1][4]
        if not cls._primary.has_key(klass): return None # プライマリポートがない
        if not map.has_key(cls._primary[klass]): return None # プライマリポートがない
        return map[cls._primary[klass]] # プライマリポートを返す
    @classmethod
    def _genport(cls,component,id,value,link,bind):
        '''
        ポートを自動生成してメソッドのバインドを試みる。
        componentが未定義であった場合にはなにもしない。
        メソッドをバインドする：
            bindが指定されている場合
            mapで指定されたプライマリポート名に対応するメソッドが存在する場合
        '''
        if component == None: return # コンポーネントが指定されていなければなにもしない
        setattr(component,id,core.Port(value)) # valueを初期値としてポートを生成する
        # ポートのリンク
        def plink(src,target):
            if src.get() == None: target.link(src,set=True)
            else: src.link(target)
        if not link == None:
            if type(link) == tuple:
                for o in link: plink(getattr(component,id),o)
            else:
                plink(getattr(component,id),link)
        # メソッドの自動バインド
        if bind == None:
            try: bind = getattr(component,'_%s' % id)
            except AttributeError: return # mapで指定されたプライマリポート名に対応するメソッドがなければなにもしない
        if type(bind) == tuple:
            for o in bind: getattr(component,id).bind(o)
        else:
            getattr(component,id).bind(bind)
    @classmethod
    def append(cls,
               klass, # ウィジェットクラス
               name=None, # ウィジェットのアクセスid
               label=None, # グリッドラベル文字列
               map=None, # ポートの結合マップ
               component=None, # ポートの制御オプション
               value=None, # ポートの制御オプション
               link=None, # ポートの制御オプション
               bind=None, # ポートの制御オプション
               geometry=None, # ジオメトリマネージャへのオプション
               label_widget=None, # ラベルウィジェットの生成オプション
               label_geometry=None, # ラベルのジオメトリマネージャへのオプション
               cnf=None, # ウィジェットの生成オプション
               *args, # ウィジェットの生成オプション
               **kwargs): # ウィジェットの生成オプション
        '''
        ウィジェットの生成ルールを登録する。
        プライマリポートへリンクするポート名を返す。プライマリポートが登録されていなければNoneを返す。
        ルールの登録が終了したらEz.Grid.finish()を呼ばなければならない。
        
        name:           ウィジェットのアクセスid。未定義ならばuuidが生成される。
        label:          グリッドラベル文字列
        map:            ポートのリンクルール{'guiportname':'componentportname'}
                        プライマリポートはnameへリンクするようにデフォルトで設定されるがmapの指定が優先。
        widget:         ウィジェット生成オプション。masterは無効。
        geometry,       
        label_widget,   
        label_geometry: gui.Grid.appendにバイパス
        *args,          
        **kwargs:       ウィジェット生成オプション。masterは無効。
        
        component:      ポートを生成したいコンポーネントインスタンス
                        指定するとプライマリポートとリンクするポートの自動生成、メソッドのバインドと他のポートとのリンクを試みる。
                        プライマリポートが登録されていない場合にはなにもしない。
                        生成するポートの属性名はmapで指定された値に、未指定ならばウィジェットのアクセスidになる。
        value:          生成するポートの初期値
        bind:           ポートに結合するメソッド（methodまたは(method1,method2,...)）
                        bindで指定したメソッドを自動生成したポートとただちにバインドする。
                        bindで定義したメソッドが存在しない場合にはエラーを返す。
                        bindが未定義ならば'_プライマリポート名'のメソッドとのバインドを試みる。
                        このとき、メソッドが存在しない場合にはなにもしない。
        link:           ポートにリンクするポート(portまたは(port1,port2,...))
                        linkで指定したポートを自動生成したポートとただちにリンクする。
                        linkで指定したポートが存在しない場合にはエラーを返す。
        
        self.Component.append(Entry,name='entry',label='entry',component=self,value=0.)
        self.Component.append(Entry,name='entry',label='entry')
        self.entry = Port(0.).bind(self._entry)
        
        self.Component.append(Widget,label='widget',map=kw(value='win',remote='wremote'),component=self)
        self.Component.append(Widget,label='widget',map=kw(value='win',remote='wremote'))
        self.win = Port(None).bind(self._win)
        '''
        cls._initialize()
        id = cls._append(klass,name,label,map,geometry,label_widget,label_geometry,cnf,*args,**kwargs)
        if not id == None: cls._genport(component,id,value,link,bind)
        return id
    @classmethod
    def finish(cls):
        cls._ezlock    = True
        cls._ezcounter = 0
    @classmethod
    def primary(cls,klass,name):
        cls._initialize()
        cls._primary[klass] = name
    def __init__(self,master=None,cnf={},**kw):
        gui.Grid.__init__(self,master,cnf,**kw)
        for klass,name,option,label,map,geometry,label_widget,label_geometry in self._ezconfig:
            args,kwargs = option
            # kwargs中のcommandを処理する（ボタンの関数割り当てなど）
            gui.Grid.append(self,klass(master=self,name=name,*args,**kwargs),label=label,geometry=geometry,label_widget=label_widget,label_geometry=label_geometry)
    def assign(self,component):
        '''
        mapのkeyをgui側、itemをcomponent側としてポートをリンクする
        '''
        for klass,name,option,label,map,geometry,label_widget,label_geometry in self._ezconfig:
            widget = self.children[name]
            for key in map:
                gui,cmp = key,map[key]
                gui,cmp = getattr(widget,gui), getattr(component,cmp)
                # もしコンポーネント側のポート値がNoneであればWidget側のデータを用いる
                if cmp.get() == None:
                    gui.link(cmp)
                else:
                    cmp.link(gui,set=True)

class Plot(Tkinter.Frame): # Grid互換
    _ezcomponent = None
    _ezcounter   = 0
    _primary = {mpl.BasePlot: 'sigin'}
    @classmethod
    def newComponent(cls,component):
        def extract_name(port):
            members = inspect.getmembers(component)
            objs    = [obj for name,obj in members]
            if objs.count(port):
                index = objs.index(port)
                name  = members[index][0]
                return name
        # ただちにインスタンスを生成する
        for port in FactoryPort.SEQUENCE:
            try:
                kwargs = FactoryPort.ITEMS[port]['widget_kwargs']
            except KeyError:
                kwargs = {}
            if kwargs.has_key('name'):
                cname = kwargs['name']
            else:
                cname = extract_name(port)
            if not component.children.has_key(cname):
                klass = FactoryPort.ITEMS[port]['widget_klass']
                FactoryPort.new_context()
                child = klass(component,name=cname)
                FactoryPort.resume()
                # プロッタのオプションを設定する
                def option(name=None,frametext=None,**kw): return kw
                for key in option(**kwargs):
                    getattr(child,key).set(kwargs[key])
    @classmethod
    def make(cls,component):
        try: cls._ezlock
        except AttributeError: pass
        else:
            # コンポーネント名が指定されていたらただちにインスタンスを生成する
            cls.newComponent(component)
            FactoryPort.reset()
            return
        def extract_name(port):
            members = inspect.getmembers(component)
            objs    = [obj for name,obj in members]
            if objs.count(port):
                index = objs.index(port)
                name  = members[index][0]
                return name
        def append(port):
            klass = FactoryPort.ITEMS[port]['widget_klass']
            name  = FactoryPort.ITEMS[port]['name']
            try:
                kwargs = FactoryPort.ITEMS[port]['widget_kwargs']
            except KeyError:
                kwargs = {}
            #
            try:
                map = FactoryPort.ITEMS[port]['map']
            except KeyError:
                map = {}
            if PRIMARY_PORTS.has_key(klass):
                if not map.has_key(PRIMARY_PORTS[klass]):
                    map[PRIMARY_PORTS[klass]] = name
            #
            cls._ezconfig.append((klass,name,map,kwargs))
        #
        cls._ezconfig = []
        # コンポーネント名が指定されていたらただちにインスタンスを生成する
        cls.newComponent(component)
        # アトリビュート名とポートオブジェクトの対応マップを生成する
        for port in FactoryPort.SEQUENCE:
            name = extract_name(port)
            if name:
                FactoryPort.ITEMS[port]['name'] = name
        # アイテムを登録する
        for port in FactoryPort.SEQUENCE:
            append(port)
        FactoryPort.reset()
        # ファクトリの受付終了
        cls._ezlock    = True
        cls._ezcounter = 0
    @classmethod
    def append(cls,klass,name=None,map=None,**kw):
        # いちばん最初の呼び出しの際に生成ルールリストを用意する
        try: cls._ezconfig
        except AttributeError: cls._ezconfig = []
        #
        # 呼び出しカウンタをインクリメントする
        cls._ezcounter += 1
        # ロックされていなければ生成ルールを追加する
        try: cls._ezlock
        except AttributeError:
            cls._ezconfig.append((klass,name,map,kw))
    @classmethod
    def finish(cls):
        cls._ezlock    = True
        cls._ezcounter = 0
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        for klass,name,map,kw in self._ezconfig:
            # nameはポート名
            # cnameはコンポーネント名とgui名
            if kw.has_key('name'):
                cname = kw['name']
            else:
                cname = name
            gui.Layout.Embed(self,klass,name=cname,text=name).pack(side=Tkinter.LEFT)
    def assign(self,component):
        def text(frametext=None,**kw): return frametext
        for klass,name,map,kw in self._ezconfig:
            # nameはポート名
            # cnameはコンポーネント名とgui名
            if kw.has_key('name'):
                cname = kw['name']
                del kw['name']
            else:
                cname = name
            # コンポーネントの割り当て
            if component.children.has_key(cname):
                child = component.children[cname]
            else:
                child = klass(component,name=cname) # 生存関係を親コンポーネントに同期した無名のコンポーネントを生成する
                def option(frametext=None,**kw): return kw
                # プロッタのオプションを設定する
                for key in option(**kw):
                    getattr(child,key).set(kw[key])
            self.children[cname].assign(child)
            # フレームラベルオプションの処理
            s = text(**kw)
            if s: self.children[cname].configure(text=s)
            # ポート間を結合する。
            if map == None: map = {} # ポートマップが未定義なら空辞書を用意
            if self._primary.has_key(klass): # プライマリーポートが既知ならば
                primary = self._primary[klass] # プライマリポート名を取得して
                if not map.has_key(primary): # mapにプライマリポートが含まれていなければ
                    map[primary] = name # nameで指定されるポートとプライマリポートをマッピングする
            for key in map:
                #dest = getattr(self.children[name].comp(),key)
                dest = getattr(child,key)
                dept = getattr(component,map[key])
                dest.set(dept.get()); dept.link(dest)
            
if __name__ == '__main__':
    class BasicSample(gui.ComponentWithGUI): # フレームワークの基本機能を使ったGUIの構築例
        class Console(Grid): pass # __metaclass__ = Meta
        def __init__(self,master=None,name=None):
            gui.ComponentWithGUI.__init__(self,master,name)
            # 
            self.Console.append(gui.Entry,label='entry',name='entryq')
            self.entryq = core.Port(0.).bind(self._entry)
            # ポートの自動生成、メソッドの自動割り当て（標準記法）
            self.Console.append(gui.Entry,label='entry',name='entry',component=self,value=0.)
            # ポートの自動生成、メソッドの指定割り当て
            self.Console.append(gui.Entry,label='entry',name='entry1',component=self,value=0.,bind=self._entry)
            # mapの指定
            self.Console.append(gui.Entry,label='entry',name='entrxy',map=core.kw(value='entry'))
            self.entry = core.Port(0.).bind(self._entry)
            # 標準機法(mapの省略)
            # 
            print self.Console.append(gui.Entry,name='1entry',label='entry',component=self,value=0.)
            #self.entry = core.Port(0.).bind(self._entry)
            print self.Console.append(gui.Entry,name='entry2',label='entry',component=self,value=0.)
            # idの自動生成、ポートの自動生成
            self.id = self.Console.append(gui.Entry,label='entry',component=self)
            # idの自動生成、ポートの自動生成、メソッドの指定割り当て（標準記法）
            self.Console.append(gui.Button,text=u'実行',component=self,bind=self._button)
            # idの自動生成
            id = self.Console.append(gui.Button,text=u'実行')
            setattr(self,id,core.Port(None).bind(self._button))
            class Test(core.Component):
                def __init__(self,master=None,name=None):
                    core.Component.__init__(self,master,name)
                    self.value  = core.Port(0.).bind(self._value)
                    self.value2 = core.Port(1.).bind(self._value2)
                    self.value3 = core.Port(2.).bind(self._value3)
                def _value(self): print 'value:', self.value.get()
                def _value2(self): print 'value2:', self.value2.get()
                def _value3(self): print 'value3:', self.value3.get()
            class MyEntry(gui.Entry): pass
            # bindオプションのテスト
            self.Console.append(gui.Entry,label='entry',component=self,bind=self._linktest)
            self.Console.append(gui.Entry,label='entry',component=self,bind=(self._linktest2,self._linktest3))
            # linkオプションの活用　無名ポートの活用幅を広げる
            test = Test(self,name='test')
            self.Console.append(gui.Entry,label='entry',component=self,link=test.value)
            self.Console.append(gui.Entry,label='entry',component=self,link=(test.value2,test.value3))
            self.Console.append(MyEntry,label='entry',component=self,link=test.value)
            self.Console.primary(MyEntry,'value')
            self.Console.append(MyEntry,label='entry',component=self,link=test.value)
            #
            self.Console.finish()
        def _entry(self):
            print 'entry'
        def _button(self):
            print 'button:', getattr(self,self.id).get()
        def _linktest(self):
            print 'linktest:', self.children['test'].value.get()
        def _linktest2(self):
            print 'linktest2:', self.children['test'].value2.get()
        def _linktest3(self):
            print 'linktest3:', self.children['test'].value3.get()
    BasicSample().destroy()
    
    class OtherSample(gui.ComponentWithGUI):
        class Console(Grid): pass
        def __init__(self,master=None,name=None):
            gui.ComponentWithGUI.__init__(self,master,name)
            self.Console.append(gui.Entry,label='entry',name='entry',component=self,value=0.)
            self.Console.finish()
        def _entry(self):
            print 'other_entry'
    OtherSample().destroy()
    
    import weakref,gc
    tk = Tkinter.Tk()
    basic = gui.Layout.Application(tk,BasicSample(core.Root()),title='basic sample',sync=False)
    gui.Layout.Application(Tkinter.Toplevel(tk),BasicSample(core.Root()))
    gui.Layout.Application(Tkinter.Toplevel(tk),BasicSample)
    gui.Layout.Application(Tkinter.Toplevel(tk),OtherSample(core.Root()))
    Tkinter.mainloop()
    basic = weakref.ref(basic)
    gc.collect()
    print 'basic:', basic



# coding: utf-8
"""\
PyLAF --- A laboratory application framework for Python
stable branch
"""

# デフォルトではフルセットインポートされる
from core import * # coreだけインポートしたい場合は import PyLAF.core で最小限のモジュールだけインポートされる
from widgets import *
from gui import *
from mpl import *
import kitchen
import thread
#from kitchen import Kitchen
#import ez as Ez



# coding: utf-8

import inspect, weakref

def isfamily(obj,cls): # インスタンスobjがクラスclsの一族であるかどうか調べる
    try:
        bases = inspect.getmro(obj.__class__) # オブジェクトのクラスツリーをリストで返す
    except AttributeError: # クラスでなければ？偽を返す
        return False
    if bases.count(cls): # クラスツリーにclsが含まれるならば
        return True
    else:
        return False

def kw(**kw): return kw

def args(*args,**kw): return args,kw

def parse_port_args(arg):
    if not type(arg) == tuple: return (arg,),{}
    if not len(arg) == 2: return arg,{}
    if not (type(arg[0]) == tuple and type(arg[1]) == dict): return arg,{}
    return args(*arg[0],**arg[1])

def test_parse():
    print parse_port_args('item')
    print parse_port_args(('item','item2','item3'))
    print parse_port_args((('item','item2','item3'),{'key1':'item1'}))

def caller():
    '''
    下記ブログに掲載されていたコードを使わせてもらった
    http://d.hatena.ne.jp/Kazumi007/20090914/1252915940
    !!!EzPortの中で使ったらコンポーネントが解放されなくなったので、要注意!!!
    '''
    try:
        framerecords = inspect.stack()
        framerecord  = framerecords[2]
        frame        = framerecord[0]
        arginfo      = inspect.getargvalues(frame)
        return arginfo.locals['self'] if 'self' in arginfo.locals else None
    finally:
        del frame
    return None

class Dummy(): pass
Wnone = weakref.ref(Dummy())



# coding: utf-8

import Tkinter
from core import Component, Port, Output

class StateCanceled(Exception): pass

class Buffer(Component):
    def __init__(self,master=None,name=None):
        Component.__init__(self,master,name)
        self.receive = Port(None).bind(self._set)
        self.send    = Output(None).bind(self.flush)
        self._last   = None
    def _set(self):
        self._last = self.receive.get()
    def flush(self):
        if not self._last == None:
            self.send.set_now(self._last)
            self._last = None
            
class Polling(Tkinter.Frame):
    def __init__(self,master=None,interval=200,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.interval = interval
        self.value    = Port(None)
        self._id      = None
        self._polling()
    def start(self):
        self._polling()
    def stop(self):
        if not self._id == None:
            self.after_cancel(self._id)
    def _polling(self):
        self.value.set_now(None)
        self._id = self.after(self.interval,self._polling)
        



from setuptools import setup, find_packages
setup(
      name='pylaf',
      version='0.3.2',
      package_dir={'pylaf' : 'src/pylaf', 'pylaf.utils' : 'src/pylaf/utils', 'pylaf.utils.core' : 'src/pylaf/utils/core', 'pylaf.utils.gui' : 'src/pylaf/utils/gui', 'pylaf.utils.mpl' : 'src/pylaf/utils/mpl', 'pylaf.recipes': 'src/pylaf/recipes'},
      packages=['pylaf','pylaf.utils','pylaf.utils.core','pylaf.utils.gui','pylaf.utils.mpl','pylaf.recipes'],
      classifiers=[
                   'Development Status :: 3 - Alpha',
                   'License :: OSI Approved :: BSD License',
                   'Programming Language :: Python',
                   'Topic :: Software Development',
                   ],
      description='Python Laboratory Application Framework',
      long_description='''\
      Python Laboratory Application Framework is a package for development applications.
      
      Requirements
      ------------
      * Python 2.4 or later (not support 3.x)
      
      Features
      --------
      * Add FDTD framework
      '''
      ,
      keywords=['framework',],
      author='Hidehisa Shiomi',
      author_email='pylaf@users.sourceforge.jp',
      url='http://pylaf.sourceforge.jp',
      license='BSD',
      )



# coding: utf-8

import inspect, types, platform, numpy, scipy
import Tkinter
import ttk #@UnresolvedImport
import vtk #@UnresolvedImport
from vtk.tk.vtkTkRenderWidget import vtkTkRenderWidget #@UnresolvedImport
from vtk.util import numpy_support #@UnresolvedImport
from vtk.util.numpy_support import numpy_to_vtk #@UnresolvedImport
from core import Component, Port, isPort
from gui  import Console, Panel, Standard

BUTTON_L = '<Button-1>'
if platform.system() == 'Darwin':
    BUTTON_R = '<Button-2>'
else:
    BUTTON_R = '<Button-3>'

def classname(obj):
    cname = repr(obj.__class__).split('.')[-1].split(' ')[0]
    if cname == '<type': cname = 'None'
    return cname
    
class RenderWidget(vtkTkRenderWidget):
    def __init__(self,*args,**kw):
        vtkTkRenderWidget.__init__(self,*args,**kw)
        self.dataset = {}
        self.volume  = {}
        self.actor   = {}
        render = vtk.vtkRenderer()
        render.SetBackground(1,1,1)
        self.GetRenderWindow().AddRenderer(render)
    def AddDataSet(self,dataset,name=None):
        if name == None: name = repr(id(dataset))
        self.dataset[name] = dataset
    def AddVolume(self,volume,name=None):
        if name == None: name = repr(id(volume))
        self.GetRenderWindow().GetRenderers().GetFirstRenderer().AddVolume(volume)
        self.volume[name] = volume
    def AddActor(self,actor,name=None):
        if name == None: name = repr(id(actor))
        self.GetRenderWindow().GetRenderers().GetFirstRenderer().AddActor(actor)
        self.actor[name] = actor
    def Render(self):
        for key in self.dataset: self.dataset[key]._sigin()
        self.GetRenderWindow().Render()
    def ResetCamera(self):
        self.GetRenderWindow().GetRenderers().GetFirstRenderer().ResetCamera()
        
class PortView(ttk.Treeview):
    def __init__(self,master=None,**kw):
        ttk.Treeview.__init__(self,master,**kw)
        self.ports = {}
        self.loop()
    def loop(self):
        self.populate()
        self.after(500,self.loop)
    def populate(self,parent=''):
        try: self.component
        except AttributeError: return
        ports = inspect.getmembers(self.component,isPort)
        my = [k for k in self.ports]
        other = [o for k,o in ports]
        for port in my:
            if not other.count(port):
                self.delete(self.ports[port][0])
        for name, port in ports:
            cname = classname(port.subject.value)
            if not self.ports.has_key(port):
                label = '%s(%s)' % (name,cname)
                id = self.insert(parent,'end',text=label)
                self.ports[port] = (id,name,cname)
            id, name, mycname = self.ports[port]
            if not mycname == cname:
                self.item(id,text='%s(%s)' % (name,cname))
                self.ports[port] = (id,name,cname)
    def config(self,component=None,**kw):
        if not component == None: self.component = component
        ttk.Treeview.config(self,**kw)
        

class ModelView(ttk.Treeview):
    class Menu(Tkinter.Menu):
        def __init__(self,master=None,cnf={},**kw):
            Tkinter.Menu.__init__(self,master,cnf,**kw)
            self.add_command(label='Property',command=master.popup_property)
    def __init__(self,master=None,name=None,**kw):
        ttk.Treeview.__init__(self,master,name=name)
        self.Menu(self,name='menu')
        self.bind(BUTTON_R, self.popup_menu)
        self.config(**kw)
        self.loop()
    def popup_menu(self,e):
        # TODO: ここにコンテキストメニューの制御コードを入れる
        return self.children['menu'].tk_popup(e.x_root,e.y_root)
    def myselection(self):
        item = self.selection()
        if    len(item) == 0: return ''
        elif  len(item) == 1: return item[0]
        else: return
    def popup_property(self):
        tag = self.myselection()
        if tag == None: return
        model = self.get_instance(tag)
        if model == None: return
        if self.number_available_ports(model) == 0: return
        Console(Tkinter.Toplevel(self),model,Standard,sync=False)
    def get_instance(self,tag):
        if tag == '': return # トップレベルならば None を返す
        parent_tag = self.parent(tag)
        if parent_tag == '': return self.get_dataset(tag)
        dataset      = self.get_dataset(parent_tag)
        target_cname = self.item(tag)['text']
        for name in dataset.children:
            model = dataset.children[name]
            if target_cname == classname(model): break
        return model
    def get_dataset(self,tag): # 対応するデータセットを取得する
        name = self.parse(self.item(tag)['text'])[0]
        return self.RenderWidget.dataset[name]
    def number_available_ports(self,instance):
        counter = 0
        members = inspect.getmembers(instance,isPort)
        for key,port in members:
            value = port.subject.value
            if (
                type(value) == types.NoneType or
                type(value) == types.InstanceType or
                type(value) == numpy.ndarray
                ): continue
            counter += 1
        return counter
    def loop(self):
        self.populate()
        self.after(500,self.loop)
    def parse(self,text):
        name,cname = text.split('(')
        cname = cname[:-1]
        return name, cname
    def populate(self):
        try: self.RenderWidget
        except AttributeError: return
        ROOT, datasets = '', self.RenderWidget.dataset
        source = [k for k in datasets]
        existing = {}
        for id in self.get_children(ROOT):
            name, cname = self.parse(self.item(id)['text'])
            existing[name] = (id,cname)
        for name in source:
            dataset = datasets[name]
            cname   = classname(dataset)
            if not existing.has_key(name):
                label    = '%s(%s)' % (name,cname)
                id       = self.insert(ROOT,'end',text=label)
                existing[name] = (id,cname)
            id, excname = existing[name]
            if not excname == cname:
                self.item(id,text='%s(%s)' % (name,cname))
            self.populate_actor(id)
    def populate_actor(self,parent):
        name, cname = self.parse(self.item(parent)['text'])
        dataset     = self.RenderWidget.dataset[name]
        existing    = {}
        for id in self.get_children(parent):
            cname = self.item(id)['text']
            existing[cname] = id
        for name in dataset.children:
            model = dataset.children[name]
            cname = classname(model)
            if not existing.has_key(cname):
                id = self.insert(parent,'end',text=cname)
    def config(self,RenderWidget=None,**kw):
        if not RenderWidget == None: self.RenderWidget = RenderWidget
        ttk.Treeview.config(self,**kw)

class VtkView(Component):
    class Plot(Panel):
        def __init__(self,master=None,cnf={},**kw):
            Panel.__init__(self,master,cnf,**kw)
            renderWidget = RenderWidget(self,name='renderwidget')
            renderWidget.grid(sticky=Tkinter.E+Tkinter.W+Tkinter.S+Tkinter.N)
            Tkinter.Button(self,text='ResetCamera',command=self.resetcamera).grid(sticky=Tkinter.W+Tkinter.E)
            self.grid_columnconfigure(0,weight=1)
            self.grid_rowconfigure(0,weight=1)
            self.grid_rowconfigure(1,weight=0)
        def render(self):
            self.RenderWidget().Render()
        def resetcamera(self):
            self.RenderWidget().ResetCamera()
        def RenderWidget(self):
            return self.children['renderwidget']
        def AddPort(self,name):
            if [k for k,v in inspect.getmembers(self)].count(name): return
            g, c = Port(None).bind(self.render), Port(None)
            setattr(self,name,g)
            setattr(self.component(),name,c)
            c.link(g)
    class Control(Panel):
        def __init__(self,master=None,cnf={},**kw):
            Panel.__init__(self,master,cnf,**kw)
            PortView(self,name='port').grid()
            ModelView(self,name='model').grid()
        def assign(self,component):
            Panel.assign(self,component)
            self.children['port'].config(component=component)
            self.children['model'].config(RenderWidget=self.sibling('plot').RenderWidget())
            
class RayCastVolume(vtk.vtkVolume):
    def _label(self):
        print self.label.get()
    def __init__(self,**kw):
        self.label = Port('Unnamed').bind(self._label)
        mapper = vtk.vtkVolumeRayCastMapper()
        mapper.SetVolumeRayCastFunction(vtk.vtkVolumeRayCastCompositeFunction())
        #
        alphaChannelFunc = vtk.vtkPiecewiseFunction()
        alphaChannelFunc.AddPoint(  0, 0.0)
        #alphaChannelFunc.AddPoint(127, 0.4)
        alphaChannelFunc.AddPoint(255, 0.4)
        #
        colorFunc = vtk.vtkColorTransferFunction()
        colorFunc.AddRGBPoint(  0,0.,0.,1.)
        colorFunc.AddRGBPoint(127,0.,1.,0.)
        colorFunc.AddRGBPoint(255,1.,0.,0.)
        #
        property = vtk.vtkVolumeProperty()
        property.SetColor(colorFunc)
        property.SetScalarOpacity(alphaChannelFunc)
        #
        self.SetMapper(mapper)
        self.SetProperty(property)
        self.config(**kw)
    def config(self,**kw):
        if kw == None: return
        if kw.has_key('Input'):
            input = kw['Input']
            input.children[repr(id(self))] = self
            self.GetMapper().SetInput(kw['Input'])
        
class OutlineActor(vtk.vtkActor):
    def __init__(self,**kw):
        self.filter = filter = vtk.vtkOutlineFilter()
        mapper = vtk.vtkPolyDataMapper()
        mapper.SetInputConnection(filter.GetOutputPort())
        self.SetMapper(mapper)
        self.config(**kw)
    def config(self,**kw):
        if kw == None: return
        if kw.has_key('Color'): self.GetProperty().SetColor(kw['Color'])
        if kw.has_key('Input'):
            input = kw['Input']
            input.children[repr(id(self))] = self
            self.filter.SetInput(input)
            
class UIntScalarStructuredPoints(vtk.vtkStructuredPoints):
    def __init__(self,**kw):
        self.children  = {}
        self.buffer    = buffer = numpy.zeros((0,0,0),dtype=numpy.uint8)
        self.sigin     = Port(buffer)
        self.olddataid = id(buffer)
        self.range     = (0.,1.)
        self.rmin      = Port(self.range[0])
        self.rmax      = Port(self.range[1])
        self.reset()
        self.config(**kw)
    def _sigin(self):
        newdata = self.sigin.get()
        if newdata == None: return
        if not self.olddataid == id(newdata): self.reset()
        buffer = self.update(newdata)
        # スケーリング
        #min, max = self.range
        min, max = self.rmin.get(), self.rmax.get()
        scipy.clip(buffer,min,max,out=buffer)
        self.buffer[:,:,:] = (255 * ((buffer + min)/max)).astype(self.buffer.dtype)
    def reset(self):
        data           = self.sigin.get()
        self.olddataid = id(data)
        z, y, x        = data.shape
        self.buffer   = numpy.zeros((z, y, x),dtype=self.buffer.dtype)
        nparray        = numpy_support.numpy_to_vtk(self.buffer.reshape(z * y * x))
        self.SetDimensions(x,y,z)
        self.SetScalarTypeToUnsignedShort()
        self.GetPointData().SetScalars(nparray)
    def config(self,**kw):
        if kw == None: return
        if kw.has_key('Spacing'): self.SetSpacing(kw['Spacing'])
        if kw.has_key('Range'):
            self.range = tuple(kw['Range'])
            self.rmin.set(self.range[0])
            self.rmax.set(self.range[1])
    # カスタマイズ
    def update(self,data):
        return data.copy()
        
class StructuredPoints(vtk.vtkStructuredPoints):
    def __init__(self,**kw):
        self.children   = {}
        self.vectorData = numpy.zeros((0,0,0,3),dtype=numpy.float64)
        self.scalarData = numpy.zeros((0,0,0),dtype=numpy.float64)
        self.olddataid  = id(self.vectorData)
        self.sigin      = Port(self.vectorData)#.bind(self._sigin)
        self.reset()
        self.config(**kw)
    def _sigin(self):
        newdata = self.sigin.get()
        if newdata == None: return
        if not self.olddataid == id(newdata): self.reset()
        self.update_vectors(self.vectorData,newdata)
        self.update_scalars()
        z, y, x, c = self.vectorData.shape
        self.GetPointData().SetVectors(numpy_support.numpy_to_vtk(self.vectorData.reshape(z * y * x, c)))
        self.GetPointData().SetScalars(numpy_support.numpy_to_vtk(self.scalarData.reshape(z * y * x)))
    def update_scalars(self):
        c = self.vectorData.shape[3]
        self.scalarData[:,:,:] = self.vectorData[:,:,:,0] ** 2
        for l in range(1,c):
            self.scalarData[:,:,:] += (self.vectorData[:,:,:,l] ** 2)
        self.scalarData[:,:,:] = numpy.sqrt(self.scalarData)
    def reset(self):
        data            = self.sigin.get()
        self.olddataid  = id(data)
        z, y, x, c      = self.shape(data)
        self.vectorData = numpy.zeros((z, y, x, c), dtype=numpy.float64)
        self.scalarData = numpy.zeros((z, y, x),    dtype=numpy.float64)
        self.SetDimensions(x, y, z)
        self.SetNumberOfScalarComponents(c)
    def config(self,**kw):
        if kw == None: return
        if kw.has_key('Spacing'): self.SetSpacing(kw['Spacing'])
    # カスタマイズ項目
    def update_vectors(self,vector,data):
        vector[:,:,:,:] = data
    def shape(self,data): return data.shape
        
class HedgeHog(vtk.vtkActor):
    def __init__(self,**kw):
        hedgehog    = vtk.vtkHedgeHog()
        lookuptable = vtk.vtkLookupTable()
        mapper      = vtk.vtkPolyDataMapper()
        mapper.SetLookupTable(lookuptable)
        mapper.SetInputConnection(hedgehog.GetOutputPort())
        mapper.ScalarVisibilityOn()
        #hhogMapper.SetScalarRange(0.,.01)
        #
        self.SetMapper(mapper)
        self.hedgehog = hedgehog
        self.config(**kw)
    def config(self,**kw):
        if kw == None: return
        if kw.has_key('ScaleFactor'): self.hedgehog.SetScaleFactor(kw['ScaleFactor'])
        if kw.has_key('ScalarRange'): self.GetMapper().SetScalarRange(kw['ScalarRange'])
        if kw.has_key('Input'):
            input = kw['Input']
            input.children[repr(id(self))] = self
            self.hedgehog.SetInput(input)
        #self.SetScalarRange(dataset.GetScalarRange())
        



# coding: utf-8

import weakref, inspect, platform, types, re, functools, threading, Queue, time
import utils.gui, core, utils.functions
from functools import partial
from utils.gui.tkwidgets import *
from scipy import log10

def classname(obj): return repr(obj.__class__).split('.')[-1].split(' ')[0]

class Projection:
    class Through:
        @classmethod
        def forward(cls,x):  return x
        @classmethod
        def backward(cls,x): return x
    class INV:
        @classmethod
        def forward(cls,x):  return 1./x
        @classmethod
        def backward(cls,x): return 1./x
    class DB:
        @classmethod
        def forward(cls,x):  return 10.*log10(x)
        @classmethod
        def backward(cls,x): return 10.**(x/10.)
        
class Entry(Tkinter.Entry):
    def __init__(self,master=None,format='%s',range=None,projection=Projection.Through,cnf={},**kw):
        Tkinter.Entry.__init__(self,master,cnf,**kw)
        self.projection = projection
        self.format = format
        self.old = None
        self.range = range
        self.value = core.Port('').bind(self._forward)
        self.bind('<Escape>',self._forward) # ESCキーが押されたら値を破棄する
        self.bind('<Return>',self._backward) # RETURNキーが押されたら値を確定する
        self.bind('<FocusOut>',self._forward) # FOCUSが失われたら値を確定する
    def _forward(self,*args):
        self.delete(0,len(self.get()))
        self.insert(0,self.format % self.projection.forward(self.value.get()))
    def _backward(self,*args):
        old = self.projection.forward(self.value.get())
        try: # 文字列を変換できればその値を
            new = type(self.value.get())(self.get())
        except ValueError: # できなければ零を代入する
            new = type(self.value.get())('0')
        if self.range:
            if not self.range(new): new = old # もし範囲に入っていなければ以前の値に戻す
        self.value.set(self.projection.backward(new)) # Portで保持しているオブジェクト型に変換してセット

class EntryTable(Tkinter.Frame):
    def __init__(self,master=None,row=1,column=1,format='%s',rowlabel=None,columnlabel=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.value = core.Port([['' for c in range(column)] for r in range(row)]).bind(self._forward)
        self.joint = []
        self.row   = row
        self.column = column
        # カラムラベルの生成
        if columnlabel:
            if type(columnlabel) == str:
                for j in range(column): Label(self,text=re.sub('%d','%d' % (j+1),columnlabel)).grid(row=0,column=j+1)
            else:
                for j,label in enumerate(columnlabel): Label(self,text=label).grid(row=0,column=j+1)
        # ローラベルの生成
        if rowlabel:
            if type(rowlabel) == str:
                for i in range(row): Label(self,text=re.sub('%d','%d' % (i+1),rowlabel)).grid(row=i+1,column=0)
            else:
                for i,label in enumerate(rowlabel): Label(self,text=label).grid(row=i+1,column=0)
        # エントリーウィジェットの生成
        for i in range(row):
            joint = []
            for j in range(column):
                joint.append(core.Port('').bind(functools.partial(self._backward,i=i,j=j)))
                w = Entry(self,name='item%d%d' % (i,j),format=format)
                w.grid(row=i+1,column=j+1,sticky=Tkinter.W+Tkinter.E)
                joint[-1].link(w.value)
            self.joint.append(joint)
    def _forward(self):
        self._forward_called = True
        value = self.value.get()
        for i in range(self.row):
            for j in range(self.column):
                if i >= len(value) or j >= len(value[0]):
                    self.joint[i][j].set(0)
                else:
                    self.joint[i][j].set(value[i][j])
        del self._forward_called
    def _backward(self,i,j):
        try: self._forward_called
        except AttributeError:
            value = self.value.get()
            if i < len(value) and j < len(value[0]):
                value[i][j] = self.joint[i][j].get()
                self.value.set(value)
        
class MultipleEntries(Tkinter.Frame):
    def __init__(self,master=None,name=None,column=2,projection=None,frameoption={},cnf={},**kw):
        Tkinter.Frame.__init__(self,master,name=name,cnf=frameoption)
        self.value      = core.Port(['' for i in range(column)]).bind(self._forward)
        self.mode       = core.Port(None).bind(self._forward)
        self.joint      = []
        self.column     = column
        self.projection = projection
        for i in range(column):
            self.joint.append(core.Port('').bind(self._backward))
            w = Entry(self,name='item%d' % i,cnf=cnf,**kw)
            w.grid(row=0,column=i)
            self.joint[-1].link(w.value)
    def _forward(self):
        value = self.value.get()[:self.column]
        if self.projection:
            forward = self.projection[self.mode.get()][0]
            value = forward(value)
        for o,v in zip(self.joint,value): o.set(v)
    def _backward(self):
        value    = [o.get() for o in self.joint]
        if self.projection:
            backward = self.projection[self.mode.get()][1]
            value    = backward(value)
        self.value.set(value)
        
class Label(Tkinter.Label):
    def __init__(self,master=None,format='%s',cnf={},**kw):
        Tkinter.Label.__init__(self,master,cnf,**kw)
        self.format = format
        self.value  = core.Port('').bind(self.updated)
    def updated(self):
        self.configure(text=self.format % self.value.get())

class LabelTable(Tkinter.Frame):
    def __init__(self,master=None,name=None,row=1,column=1,rowlabel=None,columnlabel=None,width=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,name=name)
        self.value = core.Port([['' for c in range(column)] for r in range(row)]).bind(self._forward)
        self.joint = []
        self.row   = row
        self.column = column
        # カラムラベルの生成
        if columnlabel:
            if type(columnlabel) == str:
                for j in range(column): Label(self,text=re.sub('%d','%d' % (j+1),columnlabel)).grid(row=0,column=j+1)
            else:
                for j,label in enumerate(columnlabel): Label(self,text=label).grid(row=0,column=j+1)
        if rowlabel:
            if type(rowlabel) == str:
                rowlabel = [re.sub('%d','%d' % (i+1),rowlabel) for i in range(row)]
            for i,label in enumerate(rowlabel):
                w = Label(self,text=label)
                w.grid(row=i+1,column=0)
                w.configure(anchor=Tkinter.W)
                if width: w.configure(width=width[0])
        for i in range(row):
            joint = []
            for j in range(column):
                joint.append(core.Port(''))#.bind(self._backward))
                w = Label(self,name='item%d%d' % (i,j),cnf=cnf,**kw)
                w.grid(row=i+1,column=j+1,sticky=Tkinter.W+Tkinter.E)
                w.configure(anchor=Tkinter.W)
                if width: w.configure(width=width[j+1])
                joint[-1].link(w.value)
            self.joint.append(joint)
    def _forward(self):
        value = self.value.get()
        for i in range(self.row):
            for j in range(self.column):
                if i >= len(value) or j >= len(value[0]):
                    self.joint[i][j].set(0)
                else:
                    self.joint[i][j].set(value[i][j])
                    
class Button(Tkinter.Button):
    '''クリックすると登録したコールバックを起動する。コールバックは通常のTkinter.Buttonと同様にcommand=<method>オプションで指定できる。'''
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.value = core.Port(None)
        self.configure(command=lambda:self.value.set(self.value.get())) # ボタンへのコールバックを設定する
        
class ToggleButton(Tkinter.Button):
    def __init__(self,master=None,text=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.value = core.Port(0)
        self.configure(command=self._clicked)
        if not type(text) == str:
            self.configure(text=text[0])
            self.text = text
        else:
            self.configure(text=text)
    def _clicked(self):
        if (self.value.get() + 1) < len(self.text):
            value = self.value.get() + 1
        else:
            value = 0
        self.configure(text=self.text[value])
        self.value.set(value)

class TrigButton(Tkinter.Button):
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.value = core.Port(None)
        self.configure(command=self._backward)
    def _backward(self):
        self.configure(state=Tkinter.DISABLED)
        self.value.set(None)
        self.configure(state=Tkinter.NORMAL)

class LatchButton(Tkinter.Button):
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.value = core.Port(True); self.value.bind(self._forward)
        self.configure(command=self._backward)
    def _backward(self):
        self.configure(state=Tkinter.DISABLED)
        self.value.set(False)
    def _forward(self):
        if self.value.get():
            self.configure(state=Tkinter.NORMAL)
        else:
            self.configure(state=Tkinter.DISABLED)
        
class Radiobuttons(Tkinter.Frame,utils.gui.SyncVarPort):
    def __init__(self,master=None,modes=[],cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.var = v = Tkinter.StringVar()
        for text,key in modes: Tkinter.Radiobutton(self,text=text,variable=v,value=key,command=self._backward).pack(anchor=Tkinter.W)
        self.value = core.Port(None); self.value.bind(self._forward)
        
class Checkbutton(Tkinter.Frame,utils.gui.SyncVarPort):
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.var = v = Tkinter.IntVar()
        Tkinter.Checkbutton(self,variable=v,command=self._backward).pack(anchor=Tkinter.W)
        self.value = core.Port(0).bind(self._forward)

class Menu(Tkinter.Menu):
    TYPE_CASCADE, TYPE_ITEM = range(2)
    ITEMS = []
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Menu.__init__(self,master,cnf,**kw)
        self.callback = []
        self.make()
    def make(self,items=None):
        if not items: items = self.__class__.ITEMS
        for child in items:
            self.add_child(self,child)
    def remove_empty_items(self,cascade):
        length = cascade.len()
        for index in range(length):
            if cascade.type(index) == 'cascade':
                label = cascade.entrycget(index,'label')
                menu  = cascade.nametowidget(cascade.entrycget(index,'menu'))
                self.remove_empty_items(menu)
                if utils.isfamily(menu,ChildrenMenu):
                    continue
                if menu.len() == 0:
                    cascade.delete(index)
    def itemtype(self,item):
        cls = self.__class__
        if len(item) == 1: return cls.TYPE_CASCADE
        if type(item[1]) == list: return cls.TYPE_CASCADE
        if repr(type(item[1])) == '<type \'classobj\'>': return cls.TYPE_CASCADE
        if type(item[1]) == dict: return cls.TYPE_ITEM
    def parse_cascade(self,item):
        if len(item) == 1: return item[0], None
        if type(item[1]) == list: return item[0], item[1:]
        if repr(type(item[1])) == '<type \'classobj\'>': return item[0], item[1]
    def add_child(self,parent,child):
        def nocommand(command=None,**kw): return kw
        cls = self.__class__
        itemtype = self.itemtype(child) # child がどのような要素か調べる
        if itemtype == cls.TYPE_CASCADE: # child が カスケード であるならば
            label, item = self.parse_cascade(child) # label と item を抽出して
            index = self.labeltoindex(label)
            if index == None: # label が 不在 ならば
                if not repr(type(item)) == '<type \'classobj\'>':
                    cascade = Menu(parent,name=label.lower()) # あたらしいカスケード要素を生成して
                    parent.add_cascade(label=label,menu=cascade)
                    parent = cascade # 生成したカスケードを parent とする
            else:
                old = self.item(label)
                if old == None: # old が カスケード でなければ
                    self.delete(index) # 元の要素を削除して
                    if not repr(type(item)) == '<type \'classobj\'>':
                        cascade = Menu(parent,name=label.lower()) # 新しいカスケード要素で上書き
                        parent.add_cascade(label=label,menu=cascade)
                        parent = cascade
                    else:
                        pass
                else:
                    parent = old
            if   item == None:
                pass
            elif repr(type(item)) == '<type \'classobj\'>':
                parent.add_cascade(label=label,menu=item(parent,name=label.lower()))
            elif type(item) == list:
                for o in item:
                    self.add_child(parent,o)
        elif itemtype == cls.TYPE_ITEM:
            name, kwargs = child
            if kwargs.has_key('command'): # もしコマンドが定義されていて
                if type(kwargs['command']) == str: # オペランドがメソッド名ならば
                    command = getattr(self,kwargs['command']) # 自身のメソッドをコマンドに割り当てる
                    getattr(parent,name)(command=command,**nocommand(**kwargs))
                    return
            getattr(parent,name)(**kwargs)
    def assign(self,component):
        self.comp = weakref.ref(component)
        for key in self.children:
            child = self.children[key]
            if utils.functions.isfamily(child,Menu):
                child.assign(component)
    def component(self,path=None):
        if not path: return self.comp()
        ids = path.split('.')
        child = self.comp()
        for id in ids:
            child = child.children[id]
        return child
    def labeltoindex(self,label):
        index = 0
        while index == self.index(index):
            if label == self.entrycget(index,'label'):
                return index
            index = index + 1
        return
    def len(self):
        index = 0
        while index == self.index(index):
            index = index + 1
        return index
    def item(self,label):
        index = self.labeltoindex(label)
        if not index: return
        return self.nametowidget(self.entrycget(index,'menu'))

class ChildrenMenu(Menu):
    def assign(self,component):
        Menu.assign(self,component)
        self.update_children()
    def update_children(self):
        try: self.comp # もしコンポーネントが割り付けられていなければなにもしない
        except AttributeError: return
        component = self.component()
        if component == None: # もしコンポーネントが廃棄されていれば割り付けを解除する
            del self.comp
            return
        children = component.children.copy()
        # メニューに存在する子Ｃが実在するかチェックしてメニューを削除する
        existence = ['%s:%s' % (key,classname(children[key])) for key in children]
        remove = []
        for index in range(self.len()):
            label = self.entrycget(index,'label')
            flag = False
            for s in existence:
                if s == label:
                    flag = True
                    break
            if not flag:
                remove.append(index)
        for index in remove:
            self.delete(index)
        # ツリーに存在する子Ｃがメニューアイテムに存在するかチェックしてメニューアイテムを追加する
        for s in existence:
            flag = False
            for index in range(self.len()):
                label = self.entrycget(index,'label')
                if s == label:
                    flag = True
                    break
            if flag: continue
            self.add_command(label=s,command=partial(self.popup,label=s))
        self.after(500,self.update_children)
    def popup(self,label):
        key = label.split(':')[0]
        object = self.component(key)
        #
        tk = self
        while tk: tk = tk.master
        Layout.Application(Tkinter.Toplevel(tk),object,sync=False)
        
class FileMenu(Menu):
    ITEMS = []
    RESTS = [
             ['add_command', utils.kw(label='Config',command='popup_config')],
             ['add_command', utils.kw(label='Console',command='popup_console')],
             ]
    def make_rest(self):
        cls = self.__class__
        if not self.len() == 0:
            self.add_separator()
        self.make(cls.RESTS)
    def popup_config(self):
        component = self.component()
        if self.children.has_key('config'): return # すでにconfigを生成していなく
        try: component.Config # componentにConfigクラスが存在していれば
        except AttributeError: pass
        else:
            component.Config(master=Tkinter.Toplevel(self,name='config'),name='config').pack() # configパネルをポップアップする
            self.children['config'].children['config'].assign(component)
    def popup_console(self):
        component = self.component()
        if self.children.has_key('console'): return # すでにconsoleを生成していなく
        try: component.Console # componentにConsoleクラスが存在していれば
        except AttributeError: pass
        else:
            component.Console(master=Tkinter.Toplevel(self,name='console'),name='console').pack() # configパネルをポップアップする
            self.children['console'].children['console'].assign(component)

class DefaultMenu(Menu):
    DEFAULT_ITEMS = [
                     ['PyLAF', ],
                     ['File', FileMenu],
                     ['Edit'],
                     ['Children', ChildrenMenu],
                     ]
    ITEMS = []
    #RESTS = [
    #         ['Window', ],
    #         ['Help', ],
    #         ]
    # TODO:GUI毎に管理しなければならないパラメータの取り扱い。Componentからは初期値のみ管理。
    def __init__(self,master=None,desc=[],cnf={},**kw):
        cls = self.__class__
        ITEMS = cls.ITEMS
        cls.ITEMS = []
        Menu.__init__(self,master,cnf,**kw)
        self.make(cls.DEFAULT_ITEMS)
        self.make(ITEMS)
        self.make_rest(self.children)
        self.remove_empty_items(self)
    def make_rest(self,children):
        #self.make(self.__class__.RESTS)
        for key in children:
            child = children[key]
            if utils.isfamily(child,Menu):
                try: make_rest = getattr(child,'make_rest')
                except AttributeError: continue
                make_rest()
                self.make_rest(child.children)
                
class ComponentWithGUI(core.Component,utils.gui.WithGUI):
    '''
    GUIを付随するコンポーネント
    ユーザコンポーネントを拡張する際のテンプレート
    destroy()が複数回実行されるのを避けるコードを含む
    '''
    def __init__(self,master=None,name=None):
        core.Component.__init__(self,master,name)
        utils.gui.WithGUI.__init__(self)
    def destroy(self):
        if self._dying: return # すできComponentWithGUI.destroy()を実行していなければ
        self._dying = True # 循環参照判別フラグを立て
        utils.gui.WithGUI.destroy(self)
        core.Component.destroy(self) # 下層のコンポーネントを破棄する
        
class Grid(Tkinter.Frame): # パネル構成のため、グリッド状にウィジェットを配置する
    # 複数のウィジェットを並べたい場合はFrameで固めて配置する
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.__geometry       = core.kw(sticky=Tkinter.W+Tkinter.E)
        self.__label_widget   = {}
        self.__label_geometry = core.kw(sticky=Tkinter.W+Tkinter.E)
    def append(self,widget,label=None,geometry={},label_widget={},label_geometry={}):
        '''
        widgetを最終行にgridする。labelが設定されていればラベル付きでウィジェットを配置する。
        ラベルのアクセスidはlabel+row(label0,label1,...)
        geometry:ウィジェット配置オプション。row,rowspan,column,columnspanは無効。
        label_widget：ラベル生成オプション。masterは無効。
        label_geometry:ラベル配置オプション。row,rowspan,column,columnspanは無効。
        '''
        def remove_options(kwargs,*options):
            for key in options:
                if kwargs.has_key(key):
                    del kwargs[key]
            return kwargs
        def update_options(default,custom):
            for key in custom: default[key] = custom[key]
            return default
        _geometry       = update_options(self.__geometry.copy(),      remove_options(geometry,'row','rowspan','column','columnspan'))
        _label_widget   = update_options(self.__label_widget.copy(),  remove_options(label_widget,'master'))
        _label_geometry = update_options(self.__label_geometry.copy(),remove_options(label_geometry,'row','rowspan','column','columnspan'))
        col,row = self.grid_size()
        if label:
            Tkinter.Label(self,text=label,name='label%d' % row,**_label_widget).grid(row=row,column=0,**_label_geometry)
            widget.grid(row=row,column=1,**_geometry)
        else:
            widget.grid(row=row,column=0,columnspan=2,**_geometry)

class Layout:
    class Utilities:
        def widget(self,path):
            ids = path.split('.')
            child = self
            for id in ids:
                child = child.children[id]
            return child
        def component(self,path=None):
            if not path: return self.comp()
            ids = path.split('.')
            child = self.comp()
            for id in ids:
                child = child.children[id]
            return child
        def _assign(self,comp,key):
            try: self.children[key].assign
            except (KeyError,AttributeError): pass
            else: self.children[key].assign(comp)
    class Embed(Tkinter.LabelFrame,Utilities): # 埋め込み型のレイアウト。ラベルフレームで囲ってラベルを右クリックでポップアップメニュー。
        def __init__(self,master=None,cls=None,console=False,alignment='vertical',cnf={},**kw):
            Tkinter.LabelFrame.__init__(self,master,cnf,**kw)
            if alignment=='vertical':
                packw = {'side' : Tkinter.TOP}
            elif alignment=='horizontal':
                packw = {'side' : Tkinter.LEFT, 'anchor' : Tkinter.N}
            try: cls.Plot
            except AttributeError: pass
            else: cls.Plot(master=self,name='plot').pack(**packw)
            self.make_menu(cls)
        def make_menu(self,cls):
            try: cls.Menu
            except AttributeError:
                klass = DefaultMenu
            else:
                klass = cls.Menu
            menu = klass(master=self,name='menu')
            # PyLAFメニューの削除
            index = menu.labeltoindex('PyLAF')
            if not index == None:
                menu.delete(index)
            # ポップアップイベントのバインド
            self.bind('<Control-Button-1>', self._rclicked) # Ctrl+右クリック
            if platform.system() == 'Darwin': # Darwinでは右クリックが異なる
                self.bind('<Button-2>', self._rclicked) # DarwinだったらB2
            else:
                self.bind('<Button-3>', self._rclicked) # Windows,LinuxだったらB3
        def assign(self,comp):
            if comp == None: # もしコンポーネントが指定されていなければ
                class Dummy: pass
                self.comp = weakref.ref(Dummy()) # dead参照とするための捨てオブジェクトを参照
            else:
                self.comp = weakref.ref(comp) # コンポーネントを弱参照
            self._assign(comp,'plot')
            self._assign(comp,'menu')
            return self
        def popup_config(self):
            if self.children.has_key('config'): return # すでにconfigを生成していなく
            try: self.comp().Config # componentにConfigクラスが存在していれば
            except AttributeError: pass
            else:
                self.comp().Config(master=Tkinter.Toplevel(self,name='config'),name='config').pack() # configパネルをポップアップする
                self.children['config'].children['config'].assign(self.comp())
            if self.children.has_key('console'): return # すでにconsoleを生成していなければ
            else:
                self.comp().Console(master=self.children['config'].children['config'],name='console').pack()
                self.children['config'].children['config'].children['console'].assign(self.comp())
        def _rclicked(self,e):
            self.children['menu'].tk_popup(e.x_root,e.y_root)
    class EmbedWithConsole(Embed,Utilities):
        def __init__(self,master=None,cls=None,alignment='vertical',cnf={},**kw):
            Tkinter.LabelFrame.__init__(self,master,cnf,**kw)
            if alignment=='vertical':
                packw = {'side' : Tkinter.TOP}
            elif alignment=='horizontal':
                packw = {'side' : Tkinter.LEFT, 'anchor' : Tkinter.N}
            try: cls.Plot
            except AttributeError: pass
            else: cls.Plot(master=self,name='plot').pack(**packw)
            try: cls.Console
            except AttributeError: pass
            else: cls.Console(master=self,name='console').pack(**packw)
            self.make_menu(cls)
        def assign(self,comp):
            if comp == None: # もしコンポーネントが指定されていなければ
                class Dummy: pass
                self.comp = weakref.ref(Dummy()) # dead参照とするための捨てオブジェクトを参照
            else:
                self.comp = weakref.ref(comp) # コンポーネントを弱参照
            self._assign(comp,'plot')
            self._assign(comp,'console')
            self._assign(comp,'menu')
            return self
    class Application(Tkinter.Frame,Utilities): # 親はToplevelかTkのどちらかでなければならない
        def __init__(self,master=None,klass=None,title=None,sync=True,pack=True,alignment='vertical',cnf={},**kw):
            Tkinter.Frame.__init__(self,master,cnf,**kw)
            
            if type(klass) == types.InstanceType:
                instance = klass
                klass = klass.__class__
                
            if alignment=='vertical':
                packw = {'side' : Tkinter.TOP}
            elif alignment=='horizontal':
                packw = {'side' : Tkinter.LEFT, 'anchor' : Tkinter.N}
                
            try: klass.Plot # プロットクラスが存在するか確認し
            except AttributeError: pass # 存在しなければなにもしない
            else: klass.Plot(master=self,name='plot').pack(**packw) # プロットパネルの生成
            
            try: klass.Console
            except AttributeError: pass # 用意されていなければなにもしない
            else: klass.Console(master=self,name='console').pack(**packw) # コンソールパネルの生成
            
            try: klass.Status
            except AttributeError: pass # 用意されていなければなにもしない
            else: klass.Status(master=self,name='status').pack() # ステータスパネルの生成
            
            # メニューの生成
            try: klass.Menu
            except AttributeError:
                self.master.config(menu=DefaultMenu(master=self,name='menu'))
            else:
                self.master.config(menu=klass.Menu(master=self,name='menu'))
            
            self._sync  = sync # コンポーネントと寿命を同期する場合はTrue
            self._dying = False # 循環実行回避用フラグ
            
            # 後処理
            try: instance # もしklass引き数にインスタンスが設定されていれば
            except UnboundLocalError: pass
            else: self.assign(instance) # インスタンスを割り当てる
            if title: self.master.title(title) # タイトル引き数に応じてタイトルを設定する
            if pack: self.pack() # パック引き数に応じてパックする
        def assign(self,comp):
            if comp == None: # もしコンポーネントが指定されていなければ
                class Dummy: pass
                self.comp = weakref.ref(Dummy()) # dead参照とするための捨てオブジェクトを参照
            else:
                self.comp = weakref.ref(comp) # コンポーネントを弱参照
                comp._guis[repr(id(self))] = self # gui寿命同期用弱参照辞書へ登録
            self._assign(comp,'plot')
            self._assign(comp,'console')
            self._assign(comp,'status')
            self._assign(comp,'menu')
            return self
        def popup_config(self):
            if self.children.has_key('config'): return # すでにconfigを生成していなく
            try: self.comp().Config # componentにConfigクラスが存在していれば
            except AttributeError: pass
            else:
                self.comp().Config(master=Tkinter.Toplevel(self,name='config'),name='config').pack() # configパネルをポップアップする
                self.children['config'].children['config'].assign(self.comp())
        def destroy(self):
            if self._dying: return # Application.destroy()を最初に実行するのならば
            self._dying = True # 循環参照判別フラグを立て
            try: self.comp
            except AttributeError:
                Tkinter.Frame.destroy(self)
                return
            comp = self.comp()
            if not comp == None: # コンポーネントが存在して
                if self._sync: # 寿命同期スイッチがTrueであり
                    if comp._dying == True: # コンポーネント側からの呼び出しならば
                        #if self.master.children.has_key(self._name): del self.master.children[self._name] # 再呼び出しを防止し（不要？）
                        self.master.destroy() # Toplevelを削除する
                    else: # destroy元がGUI側ならば
                        comp.destroy() # コンポーネントのdestroy()メソッドをコールする。
            Tkinter.Frame.destroy(self) # 下層のウィジェットを破棄する
            
class Thread(threading.Thread):
    EVENT_START, EVENT_CANCEL, EVENT_DONE,\
    STATE_DONE, STATE_COMPUTING, STATE_CANCELED, STATE_RUNNING = range(7)
    class StateCanceled(Exception): pass
    class SafeBuffer(Tkinter.Frame):
        def __init__(self,master=None,interval=200,cnf={},**kw):
            Tkinter.Frame.__init__(self,master,cnf,**kw)
            self.receive   = core.Port(None).bind(self._set)
            self.send      = core.Port(None)
            self.interval  = core.Port(interval)
            self._last     = None
            self._polling()
        def _set(self):
            self._last = self.receive.get()
        def _polling(self):
            if not self._last == None:
                self.send.set(self._last)
                self._last = None
            self.after(self.interval.get(),self._polling)
    class SafeQueue(Tkinter.Frame):
        def __init__(self,master=None,cnf={},**kw):
            Tkinter.Frame.__init__(self,master,cnf,**kw)
            self.receive = core.Port(None).bind(self._set)
            self.send    = core.Port(None)
            self._queue  = Queue.Queue()
            self._polling()
        def _set(self):
            self._queue.put(self.receive.get())
        def _polling(self):
            try:
                while 1:
                    self.send.set(self._queue.get_nowait())
            except Queue.Empty: pass
            self.after(200,self._polling)
    class LatchedTriggerButton(Tkinter.Button):
        def __init__(self,master=None,cnf={},**kw):
            Tkinter.Button.__init__(self,master,cnf,**kw)
            self.thread  = core.Port(None)
            self.remote  = core.Port(None).bind(self._forward)
            self._thread = threading.Thread(target=self._run)
            self._state  = False
            self.configure(command=self._backward)
        def _run(self):
            self.thread.set(Thread.EVENT_START)
            self.remote.set(Thread.EVENT_DONE)
        def _backward(self):
            if not self._state: self.activate()
            else              : self.deactivate()
        def _forward(self):
            remote = self.remote.get()
            if   remote == Thread.EVENT_DONE:   self.deactivate()
            elif remote == Thread.EVENT_START:  self.activate()
            elif remote == Thread.EVENT_CANCEL: self.deactivate()
        def activate(self):
            self.configure(state=Tkinter.DISABLED)
            self._state = True
            self._thread.start()
        def deactivate(self):
            self.configure(state=Tkinter.NORMAL)
            self._state = False
            self._thread = threading.Thread(target=self._run)
    class TriggerButton(Tkinter.Button):
        def __init__(self,master=None,text=['Waiting','Running'],cnf={},**kw):
            Tkinter.Button.__init__(self,master,cnf,**kw)
            self.thread  = core.Port(None)
            self.cancel  = core.Port(None)
            self.remote  = core.Port(None).bind(self._forward)
            self._thread = threading.Thread(target=self._run)
            self._state  = False
            self._text   = text
            self.configure(command=self._backward)
            self.configure(text=text[0])
        def _run(self):
            self.thread.set(Thread.EVENT_START)
            self.cancel.set(Thread.EVENT_DONE)
            self.remote.set(Thread.EVENT_DONE)
        def _backward(self):
            if not self._state: self.activate()
            else:
                self.cancel.set(Thread.EVENT_CANCEL)
                self.deactivate()
        def _forward(self,remote=None):
            if remote == None: remote = self.remote.get()
            if   remote == Thread.EVENT_DONE:  self.deactivate()
            elif remote == Thread.EVENT_START: self.activate()
            elif remote == Thread.EVENT_CANCEL:
                self.cancel.set(Thread.EVENT_CANCEL)
                self.deactivate()
        def activate(self):
            self.configure(text=self._text[1])
            self._state = True
            self._thread.setDaemon(True)
            self._thread.start()
            self.cancel.set(Thread.STATE_RUNNING)
        def deactivate(self):
            self.configure(text=self._text[0])
            self._state = False
            self._thread = threading.Thread(target=self._run)
        
if __name__ == '__main__':
    class BasicSample(ComponentWithGUI): # フレームワークの基本機能を使ったGUIの構築例
        class Console(Grid):
            def __init__(self,master=None,cnf={},**kw):
                Grid.__init__(self,master,cnf={},**kw)
                self.append(Entry(self,name='param1'),label='A =')
                self.append(Label(self,name='param2'),label='2 x A =',label_geometry=core.kw(sticky=Tkinter.E))
                self.append(WeakValuedButton(self,name='callback',text='callback'))
                self.append(WeakValuedButton(self,name='destroy',text='destroy'))
                self.append(Tkinter.Button(self,name='cwtest',text='cwtest'))
                self.append(Tkinter.Button(self,name='rebind',text='rebind',command=core.CallWrapper(self._rebind)))
            def assign(self,component):
                self.children['param1'].value.set(component.param1.get())
                component.param1.link(self.children['param1'].value)
                self.children['param2'].value.set(component.param2.get())
                component.param2.link(self.children['param2'].value)
                self.children['callback'].configure(command=component.cb1)
                self.children['destroy'].configure(command=component.destroy)
                #self.children['cwtest'].configure(command=component.cb1) # 直接バインドするとコンポーネントが解放されない
                self.children['cwtest'].configure(command=core.CallWrapper(component.cb1))
            def _rebind(self):
                # self.children['destroy'].click() # <- こんな感じで操作できるといいね
                self.assign(BasicSample(core.Root()))
        def __init__(self,master=None,name=None):
            ComponentWithGUI.__init__(self,master,name)
            self.param1 = core.Port(1.); self.param1.bind(self.calc)
            self.param2 = core.Port(0.)
        def calc(self): # parm1に結合されたコールバック
            self.param2.set(2 * self.param1.get())
        def cb1(self): # とりあえず、何にも結合されていないコールバック
            print 'alive'
            
    tk = Tkinter.Tk()
    basic = Layout.Application(tk,BasicSample(core.Root()),title='basic sample',sync=False)
    Tkinter.mainloop()
    basic = weakref.ref(basic)
    print 'basic:', basic
